function graphik(action)

% Programm fuer das 2D Ising Modell   
% 
% Author:  Winfried Kernbichler,Dieter Mayer
% Version: 1.0 
% Date:    15. November 2001
% Status:  erste Schritte
% Missing:  
         

% Test of all gui_routines

% ==========================================================================
% ==========================================================================
% one should keep this at the beginning of the file
% just change func_name
func_name = 'graphik';                    % function name - same as declaration
% ==========================================================================
handle_name = [func_name,'_HANDLE'];  % name of global variable, erzeugt den String: gol_Handle
eval(['global ',handle_name]);        % set it global
if nargin == 0, action = 'initgraph'; end  % set to initgraph without input, wenn keine Eingabeparameter existieren
if ~strcmp(action,'initgraph'),       % vergleicht die Action mit dem Wort initgraph
  eval(['HANDLE=',handle_name,';']);  % copy global to local
else                                  % wenn die Aktion initgraph ist
  % one can change this strings, Grundeinstellungen
  logo_switch   = 1;                  % logo: yes (1), no (0)
  fig_title     = 'Funktionenbetrachter';
  logo_title    = 'Funktionenbetrachter';
  logo_name     = 'Winfried Kernbichler, Dieter Mayer';
  logo_lv       = 'Multimediale Lehre';
  logo_type     = 'logo_1';           % only this is defined at the moment
  min_farbe     = 0;                  % Minimalwert der rot, gruen und blau Anteile von Farben
  max_farbe     = 1;                  % Maximalwert der rot, gruen und blau Anteile von Farben
  min_dicke     = 1;                  % Minimalwert fuer die Liniendicke
  max_dicke     = 5;                  % Maximalwert fuer die Liniendicke
  sta_dicke     = 2;                  % Standardwert fuer die Liniendicke
  sta_rot       = 0;                  % Standardeinstellung fuer rot
  sta_gruen     = 0.7;                % Standardeinstellung fuer gruen
  sta_blau      = 0;                  % Standardeinstellung fuer blau
  min_lim       = -10;                % untere x-lim und y-lim Werte
  max_lim       = 10;                 % obere x-lim und y-lim Werte
 
  funktion_2d   = 'sin(x)';           % Funktion fuer 2D Funktions Edit Feld
  funktion_3d   = 'cos(sin(cos(x)+sin(y)))';          % Funktion fuer 3D Funktions Edit Feld 
  
  resolution_text    = '10|20|50|100|200|500';    % Aufloesung fuer die 3D Plots
  colormap_text =      'summer|autumn|winter|spring|copper|bone|cool|hot|lines|colorcube|flag|jet|hsv|prism';        % Varianten der Colormaps fuer 3D Plots
  % some colors
  ax_color = [0.9,0.9,0.9];         % default axes color
  bo_color = [0.6,0.6,0.6];         % default border color
  % do not change
  HANDLE = [];                      % empty at the beginning
end
% ==========================================================================
% ==========================================================================
% Here every aspect of the graphics part should be handled
% This contains the definition and the callback routines
switch lower(action)
  % ========================================================================
  % initialization of program
  case 'initgraph'    
    % ----------------------------------------------------------------------
    % preprocessing for initgraph (keep this lines) 
    eval( [func_name,' ''initgraph_pre'''] );
    % ----------------------------------------------------------------------
    
    % ---------------------------------------------------------------------- 
    % definition and creation of graphics objects
    % ----------------------------------------------------------------------

    % ----------------------------------------------------------------------  
    % definition of main figure
    % ----------------------------------------------------------------------  
    F.figure = struct( ...
	'frame',    { [0.0, 0.0, 1.0, 1.0] }, ...   %Haupt-Frame soll ganzen Bildschrim ausfuellen
	'geometry', { [1] }, ...                    %nur ein Fenster
	'closerequestfcn', { 'close_request_main' }, ...        
	'renderer', { 'opengl' }, ...
	'name',     { fig_title }, ...
	'tag',      { 'main' } ...                  %Kennzeichnung, um figure wiederzuerkennen
	);
    % --- should be kept ---------------------------------------------------  
    [HANDLE,fig_han] = gui_create(gui_define(F),HANDLE); % creation
    clear F;                                      % not needed any more
    % add function name to tag and closerequestfcn
    % important for finding program related figures and to find function
    gui_add_name(fig_han,func_name,'_','b','tag');
    gui_add_name(fig_han,func_name,' ','b','closerequestfcn');
    % sets necessary userdata for figures
    gui_set_userdata(fig_han, ...
	'function',func_name, ...
	'end','exit');
    figmain_han = fig_han;
    % figure color
    fig_color = get( ...
	gui_handle_query(HANDLE.figure,'tag','*main*'),'color');
    % ----------------------------------------------------------------------
    
    % ----------------------------------------------------------------------  
    % create meta frames, Erschaffung der 5 Bildschirm
    % ----------------------------------------------------------------------  
    if logo_switch
      logo_y_pos  = 0.9; % y division between logo and rest
    else
      logo_y_pos  = 1.0;
    end
    meta_frame_logo   = [0, logo_y_pos, 1, 1-logo_y_pos]; % logo
    meta_frame_layer1 = [0, 0,          1, logo_y_pos  ]; % layer 1   
    % ----------------------------------------------------------------------  

    % ----------------------------------------------------------------------  
    % create logo
    % ----------------------------------------------------------------------
    if logo_switch
      % --------------------------------------------------------------------  
      logo_string = {logo_title,logo_name,logo_lv};
      logo_parent = figmain_han;
      % here one could use HANDLE instead of HANDLE_LOGO, then one could 
      % refere to all contents within the rest of the program.
      % When using HANDLE_LOGO this graphics part stays seperate. 
      HANDLE_LOGO = [];
      HANDLE_LOGO = gui_create_logo(HANDLE_LOGO, logo_string, ...
	  logo_parent, meta_frame_logo, logo_type);
    end
    % ----------------------------------------------------------------------  
    % ----------------------------------------------------------------------  

    % ---------------------------------------------------------------------- 
    % create main frames at 4 positions
    %  four joined parts with division lines given at 
    %  x: x_div_pos y: y_div_pos
    % ----------------------------------------------------------------------  
    mf = meta_frame_layer1;    % choose meta frame
    x_div_pos = 0.80 * mf(3);  % division line in x direction
    y_div_pos = 0.20 * mf(4);  % division line in y direction
    % frames for lower-left, lower-right, upper-left, upper-right
    ul_frame =   [ mf(1),     mf(4)/3,    mf(3)/2,   2*mf(4)/3];       % upper-left-frame
    ml_frame_1 = [ mf(1),     mf(4)/6,    mf(3)/4,     mf(4)/6];       % middle-left-frame  1                   
    ml_frame_2 = [ mf(3)/4,   mf(4)/6,    mf(3)/8,     mf(4)/6];       % middle-left-frame  2                  
    ml_frame_3 = [ 3*mf(3)/8, mf(4)/6,    mf(3)/8,     mf(4)/6];       % middle-left-frame  3                  
    ll_frame =   [ mf(1),     mf(2),      3*mf(3)/4,   mf(4)/6];       % lower-left-frame

    ur_frame_1 = [ mf(3)/2,     5*mf(4)/6, mf(3)/4,     mf(4)/6];       % upper-right-frame 1
    ur_frame_2 = [ 3*mf(3)/4,   5*mf(4)/6, mf(3)/8,     mf(4)/6];       % upper-right-frame 2
    ur_frame_3 = [ 7*mf(3)/8,   5*mf(4)/6, mf(3)/8,     mf(4)/6];       % upper-right-frame 3

    mr_frame = [ mf(3)/2,   1*mf(4)/6, mf(3)/2,   4*mf(4)/6];       % middle-right-frame
    lr_frame = [ 3*mf(3)/4, mf(2),     mf(3)/4,     mf(4)/6];       % lower-right-frame
	   
    % ----------------------------------------------------------------------  
    % definition of some basis axes system for the background of all four
    % parts defined in the meta frame.
    Fbasis.axes = struct( ...
	'geometry', { [1] }, ...
	'xborder',  { [0.0, 0.0] }, ...
	'yborder',  { [0.0, 0.0] }, ...
	'xgap',     { [0.0] }, ...
	'ygap',     { [0.0] }, ...	
	'parent',   { [1] }, ...
	'visible',  { 'on' }, ...
	'box',      { 'on' }, ...
	'xtick',    { [] }, ...
	'ytick',    { [] }, ...
	'color',    { ax_color }, ...
	'xcolor',   { bo_color }, ...
	'ycolor',   { bo_color } ...	
	);
    
    % -----------------------------------------------------------------------  
    % lower left position (beinhaltet Edit Fenster fuer die Funktionseingabe)
    % -----------------------------------------------------------------------
    % Felder fuer die 2D und 3D Funktionen
    Fbasis.axes = setfield(Fbasis.axes,'frame',ll_frame );
    Fbasis.axes = setfield(Fbasis.axes,'tag','base_axes');
    [HANDLE,newh] = gui_create(gui_define(Fbasis),HANDLE);
    gui_add_name(newh,'lower_left',' ','b','tag');            
    
    % definitions of slider values
    geom = [ 1; 1];
    
    % typical values
    visible = { 'on'; 'on'};
    str     = { funktion_2d; funktion_3d};      %Text fuer Edit Felder
%    cab = 'set_value';
    cab='set_function';

    %tag = 'slider'; Sliderbezeichnung, zugleich Variablen Name
    tag = {'fun2d';'fun3d'};%lower(str);
    % create variables from string   

    % callback action to be performed, fragt, ob bei Betaetigung 2D - oder 3D - Graphik erneuert werden muss
    cb_ac_2d = 'setup_graphik_2d'; cb_ac_3d = 'setup_graphik_3d';
    cbaction = { cb_ac_2d; cb_ac_3d };

    cbaction = transpose(flipud(lower(cbaction)));
    %cab=[func_name,' ',cbaction]
    %Struktur fuer Edit-Felder
    Fu.uicontrol = struct( ...
	'frame',         { ll_frame }, ...
	'geometry',      { geom }, ...
	'xborder',       { [0.05, 0.05] }, ...
	'yborder',       { [0.1, 0.25] }, ...
	'xgap',          { [0.05] }, ...
	'ygap',          { [0.25] }, ...
    'ysize',         { [0.10] }, ...
	'xshift',        { [0.0] }, ...	
	'yshift',        { [0.0] }, ...	
	'xmodify',       { 'size' }, ...
	'parent',        { [1] }, ...
	'style',         { 'edit' }, ...
	'visible',       { visible }, ...
	'interruptible', { 'off' }, ...
	'fontsize',      { 0.6 }, ...
	'string',        { str }, ...
	'callback',      { cab }, ...
	'tag',           { tag } ...
	);
    [HANDLE,edit_han] = gui_create(gui_define(Fu),HANDLE);
    % sets variable name in userdata + action and update information
    %gui_set_userdata(edit_han,'cbaction',cbaction);
    gui_add_name(edit_han,func_name,' ','b','callback');
    gui_add_name(edit_han,'edit',' ','e','tag');    
%    gui_add_name([edit_han],func_name,' ','b','callback');
    
    gui_set_userdata(edit_han,'cbaction',cbaction);

    str={'2D-Funktion';'3D-Funktion'};
    tag={'';''};
    % add text shifted above 
    Fu.uicontrol = setfield(Fu.uicontrol,'style',               'text');
    Fu.uicontrol = setfield(Fu.uicontrol,'yshift',               0.19);
    Fu.uicontrol = setfield(Fu.uicontrol,'horizontalalignment', 'center');    
    Fu.uicontrol = setfield(Fu.uicontrol,'string',              str);   
    Fu.uicontrol = setfield(Fu.uicontrol,'tag',              tag);   
    Fu.uicontrol = setfield(Fu.uicontrol,'callback',            '');
    Fu.uicontrol = setfield(Fu.uicontrol,'backgroundcolor',     ax_color);   
    [HANDLE,text_funk_han] = gui_create(gui_define(Fu),HANDLE);
    gui_add_name(text_funk_han,'edit_text_fun',' ','e','tag');    
    % clears structur Fu    
    clear Fu
    
    
    % ----------------------------------------------------------------------  
    % lower right position, enthaelt Hilfe und Exit Buttons
    % ----------------------------------------------------------------------  
    % basic axis for background
    Fbasis.axes = setfield(Fbasis.axes,'frame',lr_frame );
    Fbasis.axes = setfield(Fbasis.axes,'tag','base_axes');
    [HANDLE,newh] = gui_create(gui_define(Fbasis),HANDLE);
    gui_add_name(newh,'lower_right',' ','b','tag');            

    % ----------------------------------------------------------------------  
    % define uicontrol pushbutton
    geom = [1, 1 ;1, 1];
    str = reshape({'Load', 'Save'; 'Help', 'Exit'},size(geom));
    tag = lower(str);
    cab = lower(str);
    
    F.uicontrol = struct( ...
	'frame',    { lr_frame }, ...
	'geometry', { geom }, ...
	'parent',   { [1] }, ...
	'style',    { 'pushbutton' }, ...
    'xgap',     { [0.10] }, ...
	'ygap',     { [0.10] }, ...
    'xshift',   { [0.00] }, ...	
	'yshift',   { [0.00] }, ...	
	'visible',  { 'on' }, ...
	'fontsize', { 0.4 }, ...
	'string',   { str }, ...
	'callback', { cab }, ...
	'tag',      { tag } ...
	);
    [HANDLE,newh] = gui_create(gui_define(F),HANDLE); clear F;
    gui_add_name(newh,func_name,' ','b','callback');
    gui_add_name(newh,'lower_right',' ','b','tag');            
    
    % ----------------------------------------------------------------------  

    % -----------------------------------------------------------------------------  
    % middle left 1 position, enthaelt die Slider-Einstellungen fuer die 2D Graphik
    % -----------------------------------------------------------------------------
     
    Fbasis.axes = setfield(Fbasis.axes,'frame',ml_frame_1 );
    Fbasis.axes = setfield(Fbasis.axes,'tag','base_axes');
    [HANDLE,newh] = gui_create(gui_define(Fbasis),HANDLE);    
    gui_add_name(newh,'middle_left_1',' ','b','tag');  
    
    % definitions of slider values
    geom = [ 1, 1,; 1, 1];
    % typical values
    visible = { 'on', 'on';'on', 'on'  };
    minval  = { min_farbe, min_farbe;  min_farbe,   min_dicke}; % minimum
    maxval  = { max_farbe, max_farbe;  max_farbe,   max_dicke}; % maximum
    svalue  = { sta_rot, sta_gruen;  sta_blau,    sta_dicke}; % value for slider
    evalue  = { sta_rot, sta_gruen;  sta_blau,    sta_dicke}; % value for edit (string)
    for k = 1:length(svalue(:)), evalue{k} = num2str(svalue{k}); end

    %tag = 'slider'; Sliderbezeichnung, zugleich Variablen Name
    cab = 'set_value';
    str = { ...
	'Rot','Gruen'; ...
        'Blau', 'Dicke'...
	};
    tag = lower(str);
    % create variables from string
    var = transpose(flipud(lower(str))); % variables
    % here transpose and flipud is necessary to allow later use with
    % gui_add_userdata 
    % graphics obejects are counted from lower left corner along the 
    % second dimension, arrays and cells from upper left corner along
    % the first dimension.
    % The command lower is used to make sure variable names are lower
    % case.

    % callback action to be performed, fragt, ob bei Betaetigung die Graphik erneuert werden muss
    cb_ac_1 = 'setup_farbe_2d'; cb_ac_2 = 'setup_dicke_2d';
    cbaction = { cb_ac_1, cb_ac_1; cb_ac_1, cb_ac_2 };

    cbaction = transpose(flipud(lower(cbaction)));
    % what should be changed in the slider, rundet Sliderwerte auf ganze (bzw gerade) Zahlen
    cb_ch1 = { 'value','round(##)' }; cb_ch2 = { 'value','2*round(##/2)' };cb_no = {};
    cbchange_sl = { cb_no, cb_no; cb_no, cb_no}; 
    cbchange_sl = transpose(flipud(cbchange_sl));
    % what should be changed in the edit field, rundet Werte in den Edit-Feldern auf ganze bzw. gerade Zahlen
    cb_ch1 = { 'string','num2str((round(str2num(##))))' };cb_ch2 = { 'string','num2str(2*(round(str2num(##)/2)))' };cb_no = {};
    cbchange_ed = { cb_no, cb_no; cb_no, cb_no};

    cbchange_ed = transpose(flipud(cbchange_ed));
    % what should be updated when slider changes, see also updatehandle, Edit-Felder -> Slider
    update_sl = { { 'value','string','num2str(##)' } };
    update_sl = transpose(flipud(update_sl));
    % what should be updated when edit changes, see also updatehandle, Slider -> Edit-Felder    
    update_ed = { { 'string','value','str2num(##)' } };
    update_ed = transpose(flipud(update_ed));
    
    sty=reshape({ 'slider','slider','slider','slider'},size(geom));
 
%Struktur fuer Slider
    Fu.uicontrol = struct( ...
	'frame',         { ml_frame_1 }, ...
	'geometry',      { geom }, ...
	'xborder',       { [0.05, 0.05] }, ...
	'yborder',       { [0.1, 0.3] }, ...
	'xgap',          { [0.05] }, ...
	'ygap',          { [0.3] }, ...
	'xshift',        { [0.0] }, ...	
	'yshift',        { [0.0] }, ...	
	'xmodify',       { 'size' }, ...
	'parent',        { [1] }, ...
	'style',         { sty }, ...
	'min',           { minval }, ...
	'max',           { maxval }, ...
	'value',         { svalue }, ...
	'visible',       { visible }, ...
	'interruptible', { 'off' }, ...
	'fontsize',      { 0.5 }, ...
	'string',        { str }, ...
	'callback',      { cab }, ...
	'tag',           { tag } ...
	);    

    [HANDLE,slider_han] = gui_create(gui_define(Fu),HANDLE);
    % sets variable name in userdata + action and update information
    gui_set_userdata(slider_han,'variable',var); 
    gui_set_userdata(slider_han,'cbaction',cbaction);
    gui_set_userdata(slider_han,'cbchange',cbchange_sl);
    gui_set_userdata(slider_han,'updateproperty',update_sl); 
    gui_add_name(slider_han,'slider',' ','e','tag');    
    
    % add edit field shifted above (look for 'yshift')
    Fu.uicontrol = setfield(Fu.uicontrol,'xmodify',             'left');
    Fu.uicontrol = setfield(Fu.uicontrol,'xsize',               0.15);
    Fu.uicontrol = setfield(Fu.uicontrol,'yshift',              0.20);
    Fu.uicontrol = setfield(Fu.uicontrol,'style',               'edit');
    Fu.uicontrol = setfield(Fu.uicontrol,'horizontalalignment', 'left');    
    Fu.uicontrol = setfield(Fu.uicontrol,'string',              evalue);    
    Fu.uicontrol = setfield(Fu.uicontrol,'fontsize',            0.60);
    Fu.uicontrol = setfield(Fu.uicontrol,'callback',            'edit_check');
    %Fu.uicontrol = setfield(Fu.uicontrol,'tag',                 'slider_edit');
    Fu.uicontrol = setfield(Fu.uicontrol,'backgroundcolor',     ax_color);   
    [HANDLE,edit_han] = gui_create(gui_define(Fu),HANDLE);
    % sets variable name in userdata + action and update information
    gui_set_userdata(edit_han,'variable',var); 
    gui_set_userdata(edit_han,'cbaction',cbaction);
    gui_set_userdata(edit_han,'cbchange',cbchange_ed);
    gui_set_userdata(edit_han,'updateproperty',update_ed);
    gui_set_userdata(edit_han,'callback',[func_name,' ',cab]);
    gui_add_name(edit_han,'slider_edit',' ','e','tag');    

    % add text shifted above 
    Fu.uicontrol = setfield(Fu.uicontrol,'xmodify',             'right');
    Fu.uicontrol = setfield(Fu.uicontrol,'xsize',               0.20);
    Fu.uicontrol = setfield(Fu.uicontrol,'style',               'text');
    Fu.uicontrol = setfield(Fu.uicontrol,'horizontalalignment', 'right');    
    Fu.uicontrol = setfield(Fu.uicontrol,'string',              str);    
    Fu.uicontrol = setfield(Fu.uicontrol,'fontsize',            0.8);
    Fu.uicontrol = setfield(Fu.uicontrol,'callback',            '');
    %Fu.uicontrol = setfield(Fu.uicontrol,'tag',                 'slider_text');
    Fu.uicontrol = setfield(Fu.uicontrol,'backgroundcolor',     ax_color);   
    [HANDLE,text_han] = gui_create(gui_define(Fu),HANDLE);
    gui_add_name(text_han,'slider_text',' ','e','tag');    

    % updatehandle for slider (handles of edit)
    edit_cell = mat2cell(edit_han,1,ones(size(edit_han)));
    gui_set_userdata(slider_han,'updatehandle',edit_cell); 
    % updatehandle for edit (handles of slider)
    slider_cell = mat2cell(slider_han,1,ones(size(slider_han)));
    gui_set_userdata(edit_han,'updatehandle',slider_cell); 
    % adds function name to callback    
    gui_add_name([slider_han,edit_han],func_name,' ','b','callback');
    % adds common name to tag
    gui_add_name([slider_han,edit_han,text_han],'lower_left',' ','b','tag');
    % clears structur Fu    
    clear Fu
    
    
    % -----------------------------------------------------------------------------  
    % middle left 2 position, enthaelt die Edit-Einstellungen fuer die 2D Graphik
    % -----------------------------------------------------------------------------
     
    Fbasis.axes = setfield(Fbasis.axes,'frame',ml_frame_2 );
    Fbasis.axes = setfield(Fbasis.axes,'tag','base_axes');
    [HANDLE,newh] = gui_create(gui_define(Fbasis),HANDLE);    
    gui_add_name(newh,'middle_left_2',' ','b','tag');  
    
    % definitions of slider values
    geom = [ 1;1];
    % typical values
    visible = { 'on'; 'on'};
    s2value  = { min_lim; max_lim}; % value for edit (string)
    e2value=s2value;
    for k = 1:length(s2value(:)), e2value{k} = num2str(s2value{k}); end

    %tag = 'edit'; Sliderbezeichnung, zugleich Variablen Name
    cab = 'set_value';
    str = { ...
	'min_X_2d';'max_X_2d'...
	};
    tag = lower(str);
    % create variables from string
    var = transpose(flipud(lower(str))); % variables
    % here transpose and flipud is necessary to allow later use with
    % gui_add_userdata 
    % graphics obejects are counted from lower left corner along the 
    % second dimension, arrays and cells from upper left corner along
    % the first dimension.
    % The command lower is used to make sure variable names are lower
    % case.

    % callback action to be performed, fragt, ob bei Betaetigung die Graphik erneuert werden muss
    cb_ac = 'setup_graphik_2d'; cb_no = '';
    cbaction = { cb_ac; cb_ac};

    cbaction = transpose(flipud(lower(cbaction)));
       
    sty=reshape({ 'edit','edit'},size(geom));
 
%Struktur fuer Edit
    Fu.uicontrol = struct( ...
	'frame',         { ml_frame_2 }, ...
	'geometry',      { geom }, ...
	'xborder',       { [0.05, 0.05] }, ...
	'yborder',       { [0.1, 0.3] }, ...
	'xgap',          { [0.05] }, ...
	'ygap',          { [0.3] }, ...
	'xshift',        { [0.0] }, ...	
	'yshift',        { [0.0] }, ...	
	'xmodify',       { 'size' }, ...
	'parent',        { [1] }, ...
	'style',         { sty }, ...
	'visible',       { visible }, ...
	'interruptible', { 'off' }, ...
	'fontsize',      { 0.7 }, ...
	'string',        { e2value }, ...
	'callback',      { cab }, ...
	'tag',           { tag } ...
	);    

    [HANDLE,edit_han] = gui_create(gui_define(Fu),HANDLE);
    % sets variable name in userdata + action and update information
    gui_set_userdata(edit_han,'variable',var); 
    gui_set_userdata(edit_han,'cbaction',cbaction);
    gui_add_name(edit_han,'edit',' ','e','tag');    
    
    % add text shifted above 
    Fu.uicontrol = setfield(Fu.uicontrol,'xsize',               0.20);
    Fu.uicontrol = setfield(Fu.uicontrol,'yshift',              0.20);
    Fu.uicontrol = setfield(Fu.uicontrol,'style',               'text');
    Fu.uicontrol = setfield(Fu.uicontrol,'horizontalalignment', 'center');    
    Fu.uicontrol = setfield(Fu.uicontrol,'string',              str);    
    Fu.uicontrol = setfield(Fu.uicontrol,'fontsize',            0.8);
    Fu.uicontrol = setfield(Fu.uicontrol,'callback',            '');
    %Fu.uicontrol = setfield(Fu.uicontrol,'tag',                 'slider_text');
    Fu.uicontrol = setfield(Fu.uicontrol,'backgroundcolor',     ax_color);   
    [HANDLE,text_han] = gui_create(gui_define(Fu),HANDLE);
    gui_add_name(text_han,'edit_text',' ','e','tag');    
    gui_add_name([edit_han],func_name,' ','b','callback');

    % clears structur Fu    
    clear Fu
    % -----------------------------------------------------------------------------  
    % middle left 3 position, enthaelt die Checkbox-Einstellungen fuer die 2D Graphik
    % -----------------------------------------------------------------------------
     
    Fbasis.axes = setfield(Fbasis.axes,'frame',ml_frame_3 );
    Fbasis.axes = setfield(Fbasis.axes,'tag','base_axes');
    [HANDLE,newh] = gui_create(gui_define(Fbasis),HANDLE);    
    gui_add_name(newh,'middle_left_3',' ','b','tag');  
    
    % definitions of slider values
    geom = [ 1;1];
    % typical values
    visible = { 'on'; 'on'};

    %tag = 'checkbox'; Sliderbezeichnung, zugleich Variablen Name
    str = { ...
	'Grid_on';'Hold_on'...
	};
    tag = lower(str);
    % create variables from string
    var = transpose(flipud(lower(str))); % variables

    % callback action to be performed, fragt, ob bei Betaetigung die Graphik erneuert werden muss
    cb_ac_grid = 'gitter_2d'; cb_ac_hold = 'hold_on';
    cab = { cb_ac_grid; cb_ac_hold};
   
    sty=reshape({ 'checkbox','checkbox'},size(geom));
 
%Struktur fuer checkbox
    Fu.uicontrol = struct( ...
	'frame',         { ml_frame_3 }, ...
	'geometry',      { geom }, ...
	'xborder',       { [0.05, 0.05] }, ...
	'yborder',       { [0.1, 0.3] }, ...
	'xgap',          { [0.05] }, ...
	'ygap',          { [0.1] }, ...
	'xshift',        { [0.0] }, ...	
	'yshift',        { [0.0] }, ...	
	'xmodify',       { 'size' }, ...
	'parent',        { [1] }, ...
	'style',         { sty }, ...
	'visible',       { visible }, ...
	'interruptible', { 'off' }, ...
	'fontsize',      { 0.7 }, ...
	'string',        { str }, ...
	'callback',      { cab }, ...
	'tag',           { tag } ...
	);    

    [HANDLE,checkbox_han] = gui_create(gui_define(Fu),HANDLE);
    gui_add_name(checkbox_han,'checkbox',' ','e','tag');    
    gui_add_name([checkbox_han],func_name,' ','b','callback');
    
    % clears structur Fu    
    clear Fu

    % --------------------------------------------------------------------------------------------------------------  
    % upper left position, enthaelt die 2D-Graphik
    % --------------------------------------------------------------------------------------------------------------
    % basic axis for background
    

    Fbasis.axes = setfield(Fbasis.axes,'frame',ul_frame );
    Fbasis.axes = setfield(Fbasis.axes,'tag','base_axes');
    [HANDLE,newh] = gui_create(gui_define(Fbasis),HANDLE);    
    gui_add_name(newh,'upper_left',' ','b','tag');            

    % ----------------------------------------------------------------------  
    % add axes for results
    geom = [1];
    tag  = {'funk_2d'};

    F.axes = struct( ...
	'frame',    { ul_frame }, ...
	'geometry', { geom }, ...
    'xgap',     { [0.05] }, ...
	'ygap',     { [0.05] }, ...
	'parent',   { [1] }, ...
    'fontsize', {0.03},...
	'visible',  { 'on' }, ...
	'box',      { 'off' }, ...
	'tag',      { tag } ...	
	);
    [HANDLE,axes_2d_h] = gui_create(gui_define(F),HANDLE);clear F;
    gui_add_name(axes_2d_h,'result','_','b','tag');
    gui_add_name(axes_2d_h,'upper_left',' ','b','tag');
    xlabel('x');
    ylabel('y');
    grid on
    % add lines to axes (funktion_2d)
    l2d_h     = gui_handle_query(axes_2d_h,'tag','*funk_2d');
    tag       = { 'funktion_2d','funktion_2d', 'funktion_2d','funktion_2d','funktion_2d','funktion_2d',...
                  'funktion_2d','funktion_2d', 'funktion_2d','funktion_2d'};
    col       = { 'b'};
    linestyle   = { '-' };
    linewidth = { 2 }; 
    markerstyle = { 'none' };
    markersize  = {  2 };
    GUI.line  = struct( 'parent', { l2d_h }, ...
	'xdata',     { [] }, ...
	'ydata',     { [] }, ...
	'linewidth', linewidth, ...
    'linestyle',    linestyle, ...
	'marker',       markerstyle, ...
	'markersize',   markersize, ...
	'erasemode',  'normal', ...
	'color',     col, ...
	'tag',       tag ...
	);
   
    [HANDLE,line_2d_h] = gui_create(GUI,HANDLE); clear GUI
     % ----------------------------------------------------------------------  
    % middle right position, enthaelt die 3D-Graphik
    % ----------------------------------------------------------------------
    % basic axis for background
    

    Fbasis.axes = setfield(Fbasis.axes,'frame',mr_frame );
    Fbasis.axes = setfield(Fbasis.axes,'tag','base_axes');
    [HANDLE,newh] = gui_create(gui_define(Fbasis),HANDLE);    
    gui_add_name(newh,'middle_right',' ','b','tag');            

    % ----------------------------------------------------------------------  
    % add axes 
    geom = [1];
    tag  = {'funk_3d'};

    F.axes = struct( ...
	'frame',    { mr_frame }, ...
	'geometry', { geom }, ...
    'xgap',     { [0.05] }, ...
	'ygap',     { [0.05] }, ...
	'parent',   { [1] }, ...
    'fontsize', {0.03},...
	'visible',  { 'on' }, ...
	'box',      { 'off' }, ...
	'tag',      { tag } ...	
	);
  

    [HANDLE,axes_3d_h] = gui_create(gui_define(F),HANDLE);clear F;
    gui_add_name(axes_3d_h,'result','_','b','tag');
    gui_add_name(axes_3d_h,'middle_right',' ','b','tag');
    
    xlabel('x');    
    ylabel('y');
    zlabel('z');
    grid on

    % add lines to axes (funktion_3d)
    l3d_h     = gui_handle_query(axes_3d_h,'tag','*funk_3d');
    tag       = { 'funktion_3d'};
    markerstyle = { 'none' };
    markersize  = {  2 };
    GUI.surface  = struct( 'parent', { l3d_h }, ...
	'tag',              tag, ...
    'buttondownfcn',    {'rotate3d on;waitforbuttonpress;waitforbuttonpress;rotate3d off'}...
	);
    [HANDLE,surf_3d_h] = gui_create(GUI,HANDLE); clear GUI
     % ----------------------------------------------------------------------  
     
      
    % -----------------------------------------------------------------------------  
    % upper right 1 position, enthaelt die Edit-Einstellungen fuer die 3D Graphik
    % -----------------------------------------------------------------------------
     
    Fbasis.axes = setfield(Fbasis.axes,'frame',ur_frame_1 );
    Fbasis.axes = setfield(Fbasis.axes,'tag','base_axes');
    [HANDLE,newh] = gui_create(gui_define(Fbasis),HANDLE);    
    gui_add_name(newh,'upper_right_1',' ','b','tag');  
    
    % definitions of slider values
    geom = [ 1, 1;1, 1];
    % typical values
    visible = { 'on', 'on'; 'on', 'on'};
    s2value  = { min_lim, max_lim; min_lim, max_lim}; % value for edit (string)
    e2value=s2value;
    for k = 1:length(s2value(:)), e2value{k} = num2str(s2value{k}); end

    %tag = 'edit'; Sliderbezeichnung, zugleich Variablen Name
    cab = 'set_value';
    str = { ...
	'min_X_3d','max_X_3d'; 'min_Y_3d','max_Y_3d'...
	};
    tag = lower(str);
    % create variables from string
    var = transpose(flipud(lower(str))); % variables
    % here transpose and flipud is necessary to allow later use with
    % gui_add_userdata 
    % graphics obejects are counted from lower left corner along the 
    % second dimension, arrays and cells from upper left corner along
    % the first dimension.
    % The command lower is used to make sure variable names are lower
    % case.

    % callback action to be performed, fragt, ob bei Betaetigung die Graphik erneuert werden muss
    cb_ac = 'setup_graphik_3d'; cb_no = '';
    cbaction = { cb_ac, cb_ac; cb_ac, cb_ac};

    cbaction = transpose(flipud(lower(cbaction)));
       
    sty=reshape({ 'edit','edit', 'edit','edit'},size(geom));
 
%Struktur fuer Edit
    Fu.uicontrol = struct( ...
	'frame',         { ur_frame_1 }, ...
	'geometry',      { geom }, ...
	'xborder',       { [0.05, 0.05] }, ...
	'yborder',       { [0.1, 0.3] }, ...
	'xgap',          { [0.05] }, ...
	'ygap',          { [0.3] }, ...
	'xshift',        { [0.0] }, ...	
	'yshift',        { [0.0] }, ...	
	'xmodify',       { 'size' }, ...
	'parent',        { [1] }, ...
	'style',         { sty }, ...
	'visible',       { visible }, ...
	'interruptible', { 'off' }, ...
	'fontsize',      { 0.7 }, ...
	'string',        { e2value }, ...
	'callback',      { cab }, ...
	'tag',           { tag } ...
	);    

    [HANDLE,edit_3d_han] = gui_create(gui_define(Fu),HANDLE);
    % sets variable name in userdata + action and update information
    gui_set_userdata(edit_3d_han,'variable',var); 
    gui_set_userdata(edit_3d_han,'cbaction',cbaction);
    gui_add_name(edit_3d_han,'edit',' ','e','tag');    
    gui_add_name([edit_3d_han],func_name,' ','b','callback');

    % add text shifted above 
    Fu.uicontrol = setfield(Fu.uicontrol,'xsize',               0.20);
    Fu.uicontrol = setfield(Fu.uicontrol,'yshift',              0.20);
    Fu.uicontrol = setfield(Fu.uicontrol,'style',               'text');
    Fu.uicontrol = setfield(Fu.uicontrol,'horizontalalignment', 'center');    
    Fu.uicontrol = setfield(Fu.uicontrol,'string',              str);    
    Fu.uicontrol = setfield(Fu.uicontrol,'fontsize',            0.8);
    Fu.uicontrol = setfield(Fu.uicontrol,'callback',            '');
    %Fu.uicontrol = setfield(Fu.uicontrol,'tag',                 'slider_text');
    Fu.uicontrol = setfield(Fu.uicontrol,'backgroundcolor',     ax_color);   
    [HANDLE,text_3d_han] = gui_create(gui_define(Fu),HANDLE);
    gui_add_name(text_3d_han,'edit_3d_text',' ','e','tag');    

    % clears structur Fu    
    clear Fu 
     % -----------------------------------------------------------------------------  
    % upper right 2 position, enthaelt die Popup-Einstellungen fuer die 3D Graphik
    % -----------------------------------------------------------------------------
     
    Fbasis.axes = setfield(Fbasis.axes,'frame',ur_frame_2 );
    Fbasis.axes = setfield(Fbasis.axes,'tag','base_axes');
    [HANDLE,newh] = gui_create(gui_define(Fbasis),HANDLE);    
    gui_add_name(newh,'upper_right_2',' ','b','tag');  
    
    % definitions of popup values
    geom = [ 1;1];
    % typical values
    visible = { 'on'; 'on'};

    %tag = 'popup'; 
    str = { ...
	resolution_text; colormap_text...
	};
    tag = lower(str);
    % create variables from string

    % callback action to be performed, fragt, ob bei Betaetigung die Graphik erneuert werden muss
    cb_ac_style = 'setup_graphik_3d'; cb_ac_colormap = 'colormap_3d';
    cab = { cb_ac_style; cb_ac_colormap};

    cbaction = transpose(flipud(lower(cbaction)));
       
    sty=reshape({ 'popup','popup'},size(geom));
 
%Struktur fuer popup
    Fu.uicontrol = struct( ...
	'frame',         { ur_frame_2 }, ...
	'geometry',      { geom }, ...
	'xborder',       { [0.05, 0.05] }, ...
	'yborder',       { [0.1, 0.3] }, ...
	'xgap',          { [0.05] }, ...
	'ygap',          { [0.2] }, ...
	'xshift',        { [0.0] }, ...	
	'yshift',        { [0.0] }, ...	
	'xmodify',       { 'size' }, ...
	'parent',        { [1] }, ...
	'style',         { sty }, ...
	'visible',       { visible }, ...
	'interruptible', { 'off' }, ...
	'fontsize',      { 0.7 }, ...
	'string',        { str }, ...
	'callback',      { cab }, ...
	'tag',           { tag } ...
	);    

    [HANDLE,popup_han] = gui_create(gui_define(Fu),HANDLE);
    % sets variable name in userdata + action and update information
    gui_add_name(popup_han,'checkbox',' ','e','tag');    
    gui_add_name([popup_han],func_name,' ','b','callback');
     
 str={'resolution';'colormap'};
       
    % add text shifted above 
    Fu.uicontrol = setfield(Fu.uicontrol,'xsize',               0.20);
    %Fu.uicontrol = setfield(Fu.uicontrol,'ysize',               0.01);

    Fu.uicontrol = setfield(Fu.uicontrol,'yshift',              0.18);
    Fu.uicontrol = setfield(Fu.uicontrol,'style',               'text');
    Fu.uicontrol = setfield(Fu.uicontrol,'horizontalalignment', 'center');    
    Fu.uicontrol = setfield(Fu.uicontrol,'string',              str);    
    Fu.uicontrol = setfield(Fu.uicontrol,'fontsize',            0.6);
    Fu.uicontrol = setfield(Fu.uicontrol,'callback',            '');
    %Fu.uicontrol = setfield(Fu.uicontrol,'tag',                 'slider_text');
    Fu.uicontrol = setfield(Fu.uicontrol,'backgroundcolor',     ax_color);   
    [HANDLE,text_popup_han_3d] = gui_create(gui_define(Fu),HANDLE);
    gui_add_name(text_popup_han_3d,'popup_text',' ','e','tag');    

    % clears structur Fu    
    clear Fu
    
    % --------------------------------------------------------------------------------  
    % upper right 3 position, enthaelt die Checkbox-Einstellungen fuer die 3D Graphik
    % --------------------------------------------------------------------------------
     
    Fbasis.axes = setfield(Fbasis.axes,'frame',ur_frame_3 );
    Fbasis.axes = setfield(Fbasis.axes,'tag','base_axes');
    [HANDLE,newh] = gui_create(gui_define(Fbasis),HANDLE);    
    gui_add_name(newh,'upper_right_3',' ','b','tag');  
    
    % definitions of checkbox values
    geom = [ 1;1];
    visible = { 'on'; 'on'};

    str = { ...
	'Grid_on';'mesh_on'...
	};
    tag = lower(str);
    % create variables from string

    % callback action to be performed, fragt, ob bei Betaetigung die Graphik erneuert werden muss
    cb_ac_grid = 'gitter_3d'; cb_ac_mesh = 'mesh';
    cab = { cb_ac_grid; cb_ac_mesh};
       
    sty=reshape({ 'checkbox','checkbox'},size(geom));
 
%Struktur fuer checkbox
    Fu.uicontrol = struct( ...
	'frame',         { ur_frame_3 }, ...
	'geometry',      { geom }, ...
	'xborder',       { [0.05, 0.05] }, ...
	'yborder',       { [0.1, 0.3] }, ...
	'xgap',          { [0.05] }, ...
	'ygap',          { [0.1] }, ...
	'xshift',        { [0.0] }, ...	
	'yshift',        { [0.0] }, ...	
	'xmodify',       { 'size' }, ...
	'parent',        { [1] }, ...
	'style',         { sty }, ...
	'visible',       { visible }, ...
	'interruptible', { 'off' }, ...
	'fontsize',      { 0.6 }, ...
    'value',         { [1] }, ...
	'string',        { str }, ...
	'callback',      { cab }, ...
	'tag',           { tag } ...
	);    

    [HANDLE,checkbox_3d_han] = gui_create(gui_define(Fu),HANDLE);
    gui_add_name(checkbox_3d_han,'checkbox',' ','e','tag');    
    gui_add_name([checkbox_3d_han],func_name,' ','b','callback');
       
    % clears structur Fu    
    clear Fu
    
    % ----------------------------------------------------------------------  

    % ----------------------------------------------------------------------  
    % keep this here to setup first image 
    % ----------------------------------------------------------------------  
    eval( [handle_name,'=HANDLE;'] );
   
    funk_han =  HANDLE.uicontrol; 
    gitter_2d_h = gui_handle_query(funk_han,'tag','*grid_on checkbox*');
    set(gitter_2d_h,'value',1);
      
    res_3d_h = gui_handle_query(funk_han,'tag','*10*');
    set(res_3d_h,'value',4);
     
    load_h = gui_handle_query(funk_han,'tag','*load*');
    gui_set_userdata(load_h,'load_status',0);
      
    eval( [func_name,' ''setup_graphik_2d'''] );
    eval( [func_name,' ''setup_graphik_3d'''] );
    

    
    % ----------------------------------------------------------------------  
    % postprocessing for initgraph (keep this lines)
    % ----------------------------------------------------------------------  
    eval( [func_name,' ''initgraph_post'''] );
    % ----------------------------------------------------------------------  
    
    % ----------------------------------------------------------------------  
    % set main figure to visible on (keep this lines)   
    % ----------------------------------------------------------------------  
    fig_han = eval([handle_name,'.figure']);   % get content of figure
    fig_han = gui_handle_query(fig_han,'tag',['*main']);
    set(fig_han,'visible','on');          % set figures to visible
    % ======================================================================
  case 'initgraph_pre'               
    % preprocessing for initgraph (keep this lines)    
    % ======================================================================
    gui_default('set') % default values
    % close figures belonging to this program
    fig_han = get(0,'children');   
    fig_han = gui_handle_query(fig_han, ...
	'type','figure','tag',[func_name,'*']);
    delete(fig_han);
    % ======================================================================
  
  case 'initgraph_post'               
    % postprocessing for initgraph (keep this lines)   
    % ======================================================================
    gui_default('remove'); % reset defaults to MATLAB defaults
    % ======================================================================
  
  case 'exit'
  % finishing the program (keep this lines)
  % ========================================================================
    fig_han = eval([handle_name,'.figure']); % get all figures
    fig_han = fig_han(ishandle(fig_han));    % check if exist as handle
    delete(fig_han);                         % closes all figures
    clear('global',handle_name)              % deletes the global variable
    % ======================================================================
  
  case 'close_request_main'  
    % close request main (keep this lines)
    % ======================================================================
    func = gui_get_userdata(gcf,'function'); % function name
    exit = gui_get_userdata(gcf,'end');      % action for exit
    msg = { ...
	['Closing this figure exits program ',func,'.']; ...
	'Do you really want to exit?' ...
	};
    tit = 'Close Request Function';
    selection = questdlg(msg,tit,'Yes','No','Yes');
    switch selection,
      case 'Yes',
	eval( [func,' ',exit] )
      case 'No'
	return
    end
    % ======================================================================
    
  case 'close_request_help'  
    % close request for help (keep this lines)
    % ======================================================================
    func = gui_get_userdata(gcf,'function');
    exit = gui_get_userdata(gcf,'end');
    msg = { ...
	'Closing this figure is not possible.'; ...
	'Do you want to make it invisible?' ...
	};
    tit = 'Close Request Function';
    selection = questdlg(msg,tit,'Yes','No','Yes');
    switch selection,
      case 'Yes',
	eval( [func,' ',exit] )
      case 'No'
	return
    end    
    % ======================================================================
        
  case 'help'
    % calling help
    % ======================================================================
    frame = [0.1, 0.1, 0.8, 0.5];
    [HANDLE,new] = gui_help(HANDLE,frame,func_name);
    if new, eval([handle_name,'=HANDLE;']); end % export to global
    % ======================================================================
  
  case 'edit_check'
    % checks string from edit field
    % ======================================================================
    str    = get(gcbo,'string');
    oldval = get(gcbo,'value');
    minval = get(gcbo,'min');
    maxval = get(gcbo,'max');
    val = str2num(str);
    % handle all error cases
  
    if ~isempty(val)
        if val<minval
            msg = { 'Value to small!','Value set to minimum.' };
            e_han = msgbox(msg,'Edit Box','error','modal');
            set(gcbo,'string',num2str(minval));
        elseif val>maxval
            msg = { 'Value to large!','Value set to maximum.' };            
            e_han = msgbox(msg,'Edit Box','error','modal');
            %e_han = errordlg(msg,'Edit Box','on');
            set(gcbo,'string',num2str(maxval));
        end
    else
        msg = { 'Wrong value!','Value set to old value.' };
        e_han = msgbox(msg,'Edit Box','error','modal');
        %e_han = errordlg(msg,'Edit Box','on');
        if isfield(get(gcbo,'userdata'),'updatehandle')
            updatehandle   = gui_get_userdata(gcbo,'updatehandle');      
            updateproperty = gui_get_userdata(gcbo,'updateproperty');
            oldval = get(updatehandle,updateproperty{2});
        end
        set(gcbo,'string',num2str(oldval));
    end
    
    % do the real callback
    if isfield(get(gcbo,'userdata'),'callback')
        eval( gui_get_userdata(gcbo,'callback') );
    end
    % ======================================================================
    
    
  case 'set_value'
    % get values from caller 
    % ======================================================================
    e_str  = gui_get_userdata(gcbo,'variable');
    cbaction = gui_get_userdata(gcbo,'cbaction');
    figure_han = get(gcbo,'parent');
    func_name = gui_get_userdata(figure_han,'function');
    
    if isfield(get(gcbo,'userdata'),'cbchange')
      cbchange = gui_get_userdata(gcbo,'cbchange');
      if ~isempty(cbchange)
	prop = cbchange{1};
	chfunc = cbchange{2};
	e_str = [ 'get(gcbo,''',prop,''')' ];
	e_str = strrep(chfunc,'##',e_str);
	e_str = [ 'set(gcbo,''',prop,''',',e_str,');' ];
	eval( e_str );
      end
    end
    
    % do the update of another handle
    if isfield(get(gcbo,'userdata'),'updatehandle')
      updatehandle   = gui_get_userdata(gcbo,'updatehandle');      
      updateproperty = gui_get_userdata(gcbo,'updateproperty');      
      if ~isempty(updatehandle)
	from_prop = updateproperty{1};
	to_prop   = updateproperty{2};
	chfunc    = updateproperty{3};
	e_str = [ 'get(gcbo,''',from_prop,''')' ];
	e_str = strrep(chfunc,'##',e_str);
	e_str = [ 'set(updatehandle,''',to_prop,''',',e_str,');' ];
	eval( e_str );
      end
    end
    
    if ~isempty(cbaction)
      eval( [func_name,' ',cbaction] );
    end
    % ======================================================================

case 'set_function'

     funk_han =  HANDLE.uicontrol;     
     load_h = gui_handle_query(funk_han,'tag','*load*');
     gui_set_userdata(load_h,'load_status',0);

     cbaction = gui_get_userdata(gcbo,'cbaction');
     funktionen_string = get(gcbo,'string');
     
     %Falls bereits (teilweise) die richtige Notation verwendet wurde

          
     x = 1:10;  %Probe-x
     y = 1:10;  %Probe-y
     %try, 
     %    eval(['z =',funktionen_string,';']);
     %catch, 
     %    msgbox('Die Funktion kann nicht ausgefuehrt werden','Achtung','error');return
     %end
       
     try, 
         eval(['z =',funktionen_string,';']);
     catch, 
         msgbox('Die Funktion kann nicht ausgefuehrt werden, es wird versucht, sie automatisch zu korrigieren ("." vor "/", "*" und "^" einfuegen))'...
             ,'Achtung','error');
        
         pause(0.1)
         waitforbuttonpress

         funktionen_string = strrep(funktionen_string,'.*','*');
         funktionen_string = strrep(funktionen_string,'./','/');
         funktionen_string = strrep(funktionen_string,'.^','^');
     
         funktionen_string = strrep(funktionen_string,'*','.*');
         funktionen_string = strrep(funktionen_string,'/','./');
         funktionen_string = strrep(funktionen_string,'^','.^');

         try, 
             eval(['z =',funktionen_string,';']);
         catch, 
                      msgbox('Automatische Korrektur gescheitert, bitte Funktion uebepruefen'...
             ,'Achtung','error');return
         end
     end
               
     set(gcbo,'string',funktionen_string);
     
     d2_oder_d3 = get(gcbo,'tag');
     switch d2_oder_d3
     case 'fun2d edit'
         min_max_2d_h = gui_handle_query(funk_han,'tag','*_2d edit*');
         set(min_max_2d_h,'enable','on')
     case 'fun3d edit'
         min_max_3d_h = gui_handle_query(funk_han,'tag','*_3d edit*');
         resolution_3d_h = gui_handle_query(funk_han,'tag','*100*');
         set(min_max_3d_h,'enable','on')
         set(resolution_3d_h,'enable','on')
     end
     
     if ~isempty(cbaction)      
         eval( [func_name,' ',cbaction] );
     end
    
case 'setup_graphik_2d'

     ax_han = HANDLE.axes;
     fun2_ax_han = gui_handle_query(ax_han,'tag','*2d*');
     axes(fun2_ax_han)
         
     line_han = HANDLE.line;
     line_2d_h = gui_handle_query(line_han,'tag','*2d*');
     
     funk_han =  HANDLE.uicontrol; 
     load_h = gui_handle_query(funk_han,'tag','*load*');
     load_status_2d = gui_get_userdata(load_h,'load_status');
     
     switch load_status_2d
     case {0,3}     
         funk_2d_h = gui_handle_query(funk_han,'tag','*fun2d*');
         funk2d_string=get(funk_2d_h,'string');
     
         xmin_h = gui_handle_query(funk_han,'tag','*min_x_2d edit');
         xmax_h = gui_handle_query(funk_han,'tag','*max_x_2d edit');

         xmin = str2num(get(xmin_h,'string'));
         xmax = str2num(get(xmax_h,'string'));
     
         x=xmin:(xmax-xmin)/1000:xmax;
         y=eval(funk2d_string).*ones(1,length(x));
         
     case 2
         x = gui_get_userdata(load_h,'x_data');
         y = gui_get_userdata(load_h,'y_data');

         xmin = min(x);
         xmax = max(x);          
     end
         
     hold_on_2d_h = gui_handle_query(funk_han,'tag','*hold_on*');
     hold_on_off = get(hold_on_2d_h,'value');
               
     if hold_on_off         
         plot_zahl=get(line_2d_h(1),'userdata');
         set(line_2d_h(1),'userdata',plot_zahl+1)
     else
         set(line_2d_h(1),'userdata',1)
         set(line_2d_h(2:end),'xdata',[],'ydata',[])
     end

     axis auto
     xlim([xmin,xmax]);
     plot_aktuell=get(line_2d_h(1),'userdata');
     set(line_2d_h(plot_aktuell),'xdata',x,'ydata',y)
   
     eval( [func_name,' ''setup_farbe_2d''']   );
     eval( [func_name,' ''setup_dicke_2d''']   );

     save_h = gui_handle_query(funk_han,'tag','*save*');
     gui_set_userdata(save_h,'dimension_aktuell','2');

     
case 'setup_graphik_3d'
     ax_han = HANDLE.axes;
     fun3_ax_han = gui_handle_query(ax_han,'tag','*3d*');
     axes(fun3_ax_han)
    
     funk_han =  HANDLE.uicontrol; 
     funk_3d_h = gui_handle_query(funk_han,'tag','*fun3d*');
         
     load_h = gui_handle_query(funk_han,'tag','*load*');
     load_status_3d = gui_get_userdata(load_h,'load_status');
           
     switch load_status_3d
     case {0,2}
         funk3d_string=get(funk_3d_h,'string');
         resol_3d_han = gui_handle_query(funk_han,'tag','*500 checkbox*');
         resolution_wahl = get(resol_3d_han,'value');
         resolution_text = cellstr(get(resol_3d_han,'string'));
         resolution = eval(char(resolution_text(resolution_wahl)));

         xmin_h = gui_handle_query(funk_han,'tag','*min_x_3d edit');
         xmax_h = gui_handle_query(funk_han,'tag','*max_x_3d edit');
 
         ymin_h = gui_handle_query(funk_han,'tag','*min_y_3d edit');
         ymax_h = gui_handle_query(funk_han,'tag','*max_y_3d edit');

         xmin = str2num(get(xmin_h,'string'));
         xmax = str2num(get(xmax_h,'string'));
         ymin = str2num(get(ymin_h,'string'));
         ymax = str2num(get(ymax_h,'string'));
              
         xx=xmin:(xmax-xmin)/resolution:xmax;
         yy=ymin:(ymax-ymin)/resolution:ymax;

         [x,y]=meshgrid(xx,yy);
     
         z=eval(funk3d_string).*ones(length(x),length(y));
     
     case 3
         x = gui_get_userdata(load_h,'x_data');
         y = gui_get_userdata(load_h,'y_data');
         z = gui_get_userdata(load_h,'z_data');

         xmin = min(min(x));
         xmax = max(max(x));   
         ymin = min(min(y));
         ymax = max(max(y));            
         
     end
     
     surf_han = HANDLE.surface;
     surf_3d_h = gui_handle_query(surf_han,'tag','*3d*');
      
     set(surf_3d_h,'facecolor','texturemap','xdata',x,'ydata',y,'zdata',z,'cdata',z)

     xlim([xmin,xmax]);
     ylim([ymin,ymax]);
  
     save_h = gui_handle_query(funk_han,'tag','*save*');
     gui_set_userdata(save_h,'dimension_aktuell','3');
          
case 'setup_farbe_2d'
       
     line_han = HANDLE.line;
     line_2d_h = gui_handle_query(line_han,'tag','*2d*');
          
     farb_han =  HANDLE.uicontrol; 
     rot_h =   gui_handle_query(farb_han,'tag','*rot slider');
     gruen_h = gui_handle_query(farb_han,'tag','*gruen slider');
     blau_h =  gui_handle_query(farb_han,'tag','*blau slider');

     rot = get(rot_h,'value');
     gruen = get(gruen_h,'value');
     blau = get(blau_h,'value');
     
     plot_aktuell=get(line_2d_h(1),'userdata');
     set(line_2d_h(plot_aktuell),'color',[rot,gruen,blau]);

case 'setup_dicke_2d'
   
     line_han = HANDLE.line;
     line_2d_h = gui_handle_query(line_han,'tag','*2d*');
      
     dicken_han =  HANDLE.uicontrol; 
     dicke_h = gui_handle_query(dicken_han,'tag','*dicke slider');
     dicke = get(dicke_h,'value');
     
     plot_aktuell=get(line_2d_h(1),'userdata');   
     set(line_2d_h(plot_aktuell),'linewidth',dicke);
 case 'gitter_2d'
     
     ax_han = HANDLE.axes;
     fun2_ax_han = gui_handle_query(ax_han,'tag','*2d*');
     axes(fun2_ax_han)
     
     gitter_ja_nein = get(gcbo,'value');
     
     if gitter_ja_nein
         grid on
     else
         grid off
     end
     
 case 'gitter_3d'
     
     ax_han = HANDLE.axes;
     fun3_ax_han = gui_handle_query(ax_han,'tag','*3d*');
     axes(fun3_ax_han)
     
     gitter_ja_nein = get(gcbo,'value');
     
     if gitter_ja_nein
         grid on
     else
         grid off
     end
     
     
 case 'mesh'
      
     ax_han = HANDLE.axes;
     fun3_ax_han = gui_handle_query(ax_han,'tag','*3d*');
     axes(fun3_ax_han)
     
     mesh_ja_nein = get(gcbo,'value');

     if mesh_ja_nein            
         shading faceted
     else
         shading interp
     end

 case 'colormap_3d'
     
     ax_han = HANDLE.axes;
     fun3_ax_han = gui_handle_query(ax_han,'tag','*3d*');
     axes(fun3_ax_han)
     
     colormap_wahl = get(gcbo,'value');
     colormap_name = cellstr(get(gcbo,'string'));
     
     eval(['colormap',' ',char(colormap_name(colormap_wahl))])

 case 'save'
       
     cd daten
     [newfile,newpath] = uiputfile('a.dat','Save file name');
     cd ..
     
     funk_han =  HANDLE.uicontrol; 
     save_h = gui_handle_query(funk_han,'tag','*save*');
     d2_d3=gui_get_userdata(save_h,'dimension_aktuell');

     line_han = HANDLE.line;
     surf_han = HANDLE.surface;

     switch d2_d3         
     case '2'        
         line_2d_h = gui_handle_query(line_han,'tag','*2d*');
         plot_aktuell=get(line_2d_h(1),'userdata');
         x = get(line_2d_h(plot_aktuell),'xdata');
         y = get(line_2d_h(plot_aktuell),'ydata');
         matrix2d = [x', y'];
         cd daten
         save(newfile,'matrix2d','-ascii')
         cd ..
     case '3'
         surf_3d_h = gui_handle_query(surf_han,'tag','*3d*');
         x = get(surf_3d_h,'xdata');
         y = get(surf_3d_h,'ydata');
         z = get(surf_3d_h,'zdata');

         matrix3d = [x,y,z];
         cd daten
         save(newfile,'matrix3d','-ascii')
         cd ..
     end
     
     
 case 'load'
     cd daten
     [newfile,newpath] = uigetfile('*.*','Load file name');
     matrix = load([newpath,newfile]);
     cd ..
     
     funk_han =  HANDLE.uicontrol;
     funk_2d_h = gui_handle_query(funk_han,'tag','*fun2d*');
     funk_3d_h = gui_handle_query(funk_han,'tag','*fun3d*');

     load_h = gui_handle_query(funk_han,'tag','*load*');
     %Abfragen, ob 2D oder 3D
     if ~isempty(find(size(matrix)==2))
         set(funk_2d_h,'string',(newfile));
         gui_set_userdata(load_h,'load_status',2);
         if size(matrix,1)==2
             xdata = matrix(1,:);
             ydata = matrix(2,:);
         else
             xdata = matrix(:,1);
             ydata = matrix(:,2);
         end
         gui_set_userdata(load_h,'x_data',xdata);
         gui_set_userdata(load_h,'y_data',ydata);
         
         min_max_2d_h = gui_handle_query(funk_han,'tag','*_2d edit*');
         set(min_max_2d_h,'enable','inactive')
         
         eval( [func_name,' ''setup_graphik_2d'''] );
     else
        set(funk_3d_h,'string',(newfile));
        gui_set_userdata(load_h,'load_status',3);
        
        max_size = max(size(matrix));
        min_size = min(size(matrix));

        if mod(max_size,3)==0;
            if size(matrix,1)==max_size
                xdata = matrix(1:max_size/3,:);
                ydata = matrix(max_size/3+1:2*max_size/3,:);
                zdata = matrix(2*max_size/3+1:end,:);
            else
                xdata = matrix(:,1:max_size/3);
                ydata = matrix(:,max_size/3+1:2*max_size/3);
                zdata = matrix(:,2*max_size/3+1:end);
            end
        elseif mod(min_size,3)==0 & mod(max_size,3)~=0;
            if size(matrix,1)==max_size
                xdata = matrix(1:min_size/3,:);
                ydata = matrix(min_size/3+1:2*min_size/3,:);
                zdata = matrix(2*min_size/3+1:end,:);
            else
                xdata = matrix(:,1:min_size/3);
                ydata = matrix(:,min_size/3+1:2*min_size/3);
                zdata = matrix(:,2*min_size/3+1:end);
            end
        else                     
            msgbox('Die Datei kann nicht geplottet werden','Achtung','error');return
        end

        gui_set_userdata(load_h,'x_data',xdata);
        gui_set_userdata(load_h,'y_data',ydata);
        gui_set_userdata(load_h,'z_data',zdata);
        
        min_max_3d_h = gui_handle_query(funk_han,'tag','*_3d edit*');
        resolution_3d_h = gui_handle_query(funk_han,'tag','*100*');

        set(min_max_3d_h,'enable','inactive')
        set(resolution_3d_h,'enable','inactive')

         
        eval( [func_name,' ''setup_graphik_3d'''] );
     end
 case 'hold_on'
    
otherwise
    % ======================================================================
    % warnings if callback routine is not defined (keep this at the end)    
    % ======================================================================
    disp(['Callback not defined: ',action]);
    % ======================================================================

end  